// set a global value
0 => int LINEAR;
1 => int EQUALPOWER;
// set a global value
//0 => int LINEAR;
1 => int CONSTANTPOWER;

// linear panning, expect pan=[0,1]
fun void panLinear( float pan, UGen left, UGen right )
{
    // clamp to bounds
    if( pan < 0 ) 0 => pan;
    else if( pan > 1 ) 1 => pan;
    // set gains
    1-pan => left.gain;
    pan => right.gain;
}
// constant power panning, expect pan=[0,1]
fun void panPower( float pan, UGen left, UGen right )
{
    // clamp to bounds
    if( pan < 0 ) 0 => pan;
    else if( pan > 1 ) 1 => pan;
    // set gains
    Math.cos( pan * pi / 2 ) => left.gain;
    Math.sin( pan * pi / 2 ) => right.gain;
}

// pan, which=0:linear|1:constantpower
fun void panning( int which, float pan, UGen left, UGen right )
{
    if( which == LINEAR ) panLinear( pan, left, right);
    else if( which == CONSTANTPOWER ) panPower( pan, left, right );
    else <<< "[pan]: ERROR specifying which pan type!", "" >>>;
}

// determines spatial position of sound file and plays it
fun void soundClip( SndBuf theSound, UGen left, UGen right,
float startLoc, float endLoc, int whichPan )
{
    // get length of sound file
    theSound.length() => dur len;
    // start sound from the beginning
    0 => theSound.pos;
    // the initial pan value (starting point)
    startLoc => float pan;
    // end time
    now + len => time endTime;
    // start time
    now => time startTime;
    // go
    while( now < endTime )
    {
        // pan it
        panning( whichPan, pan, left, right );
        // update where the sound should be
        startLoc + (endLoc-startLoc)*(now-startTime) / (endTime-startTime) => pan;
        //print
        <<< "pan:", pan >>>;
        // update rate
        1::ms => now;
    }
    // optional tail
    300::ms => now;
}

// "car"
SndBuf car;
// read
me.dir() + "/g2.wav" => car.read;
// left and right nodes for panning
car => Gain carL => NRev rL => dac.left;
car => Gain carR => NRev rR => dac.right;
// set reverb mix
.08 => rL.mix => rR.mix;
// call in the sound
spork ~ soundClip( car, carL, carR, 0, 1, CONSTANTPOWER );
car.length() => now;
me.yield();
// last item in this program is this print statement
<<<"program end at",now/second,"seconds">>>;
// and with nothing left to do this program exits
